]> sigrok.org Git - libsigrok.git/blobdiff - device.c
sr: add sr_dev_probe_enable(), abstraction wrapper around device probes
[libsigrok.git] / device.c
index 5f8d5ed2a0f423cce66a4a1520b626ec81808fe4..be647fb248e7be173810a439fe543d41fb10d59a 100644 (file)
--- a/device.c
+++ b/device.c
@@ -19,8 +19,8 @@
 
 #include <stdio.h>
 #include <glib.h>
-#include "sigrok.h"
-#include "sigrok-internal.h"
+#include "libsigrok.h"
+#include "libsigrok-internal.h"
 
 static GSList *devs = NULL;
 
@@ -55,17 +55,17 @@ static GSList *devs = NULL;
  * TODO: Error checks?
  * TODO: Option to only scan for specific devices or device classes.
  *
* @return SR_OK upon success, SR_ERR upon errors.
  */
 SR_API int sr_dev_scan(void)
 {
        int i;
        struct sr_dev_driver **drivers;
 
-       drivers = sr_hw_list();
+       drivers = sr_driver_list();
        if (!drivers[0]) {
                sr_err("dev: %s: no supported hardware drivers", __func__);
-               return SR_ERR; /* TODO: More specific error? */
+               return SR_ERR_BUG;
        }
 
        /*
@@ -74,7 +74,7 @@ SR_API int sr_dev_scan(void)
         * of these out of the way first.
         */
        for (i = 0; drivers[i]; i++)
-               sr_hw_init(drivers[i]);
+               sr_driver_init(drivers[i]);
 
        return SR_OK;
 }
@@ -273,6 +273,38 @@ SR_API int sr_dev_probe_name_set(struct sr_dev *dev, int probenum,
        return SR_OK;
 }
 
+/**
+ * Enable or disable a probe on the specified device.
+ *
+ * @param sdi The device instance the probe is connected to.
+ * @param probenum The probe number, starting from 0.
+ * @param state TRUE to enable the probe, FALSE to disable.
+ *
+ * @return SR_OK on success, or SR_ERR_ARG on invalid arguments.
+ */
+SR_API int sr_dev_probe_enable(const struct sr_dev_inst *sdi, int probenum,
+               gboolean state)
+{
+       GSList *l;
+       struct sr_probe *probe;
+       int ret;
+
+       if (!sdi)
+               return SR_ERR_ARG;
+
+       ret = SR_ERR_ARG;
+       for (l = sdi->probes; l; l = l->next) {
+               probe = l->data;
+               if (probe->index == probenum) {
+                       probe->enabled = state;
+                       ret = SR_OK;
+                       break;
+               }
+       }
+
+       return ret;
+}
+
 /**
  * Remove all triggers set up for the specified device.
  *
@@ -283,7 +315,7 @@ SR_API int sr_dev_probe_name_set(struct sr_dev *dev, int probenum,
  * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments.
  *         If something other than SR_OK is returned, 'dev' is unchanged.
  */
-SR_API int sr_dev_trigger_clear(struct sr_dev *dev)
+SR_API int sr_dev_trigger_remove_all(struct sr_dev *dev)
 {
        struct sr_probe *p;
        unsigned int pnum; /* TODO: uint16_t? */
@@ -311,7 +343,10 @@ SR_API int sr_dev_trigger_clear(struct sr_dev *dev)
 }
 
 /**
- * Add a trigger to the specified device.
+ * Add a trigger to the specified device (and the specified probe).
+ *
+ * If the specified probe of this device already has a trigger, it will
+ * be silently replaced.
  *
  * TODO: Better description.
  * TODO: Describe valid format of the 'trigger' string.
@@ -326,41 +361,38 @@ SR_API int sr_dev_trigger_clear(struct sr_dev *dev)
  *         upon other errors.
  *         If something other than SR_OK is returned, 'dev' is unchanged.
  */
-SR_API int sr_dev_trigger_set(struct sr_dev *dev, int probenum,
-                             const char *trigger)
+SR_API int sr_dev_trigger_set(const struct sr_dev_inst *sdi, int probenum,
+               const char *trigger)
 {
-       struct sr_probe *p;
+       GSList *l;
+       struct sr_probe *probe;
+       int ret;
 
-       if (!dev) {
-               sr_err("dev: %s: dev was NULL", __func__);
+       if (!sdi)
                return SR_ERR_ARG;
-       }
-
-       /* TODO: Sanity check on 'probenum'. */
 
-       /* TODO: Sanity check on 'trigger'. */
-
-       p = sr_dev_probe_find(dev, probenum);
-       if (!p) {
-               sr_err("dev: %s: probe %d not found", __func__, probenum);
-               return SR_ERR; /* TODO: More specific error? */
+       ret = SR_ERR_ARG;
+       for (l = sdi->probes; l; l = l->next) {
+               probe = l->data;
+               if (probe->index == probenum) {
+                       /* If the probe already has a trigger, kill it first. */
+                       g_free(probe->trigger);
+                       probe->trigger = g_strdup(trigger);
+                       ret = SR_OK;
+                       break;
+               }
        }
 
-       /* If the probe already has a trigger, kill it first. */
-       g_free(p->trigger);
-
-       p->trigger = g_strdup(trigger);
-
-       return SR_OK;
+       return ret;
 }
 
 /**
  * Determine whether the specified device has the specified capability.
  *
- * TODO: Should return int?
- *
  * @param dev Pointer to the device to be checked. Must not be NULL.
- *            The device's 'driver' field must not be NULL either.
+ *            If the device's 'driver' field is NULL (virtual device), this
+ *            function will always return FALSE (virtual devices don't have
+ *            a hardware capabilities list).
  * @param hwcap The capability that should be checked (whether it's supported
  *              by the specified device).
  *
@@ -370,23 +402,32 @@ SR_API int sr_dev_trigger_set(struct sr_dev *dev, int probenum,
  */
 SR_API gboolean sr_dev_has_hwcap(const struct sr_dev *dev, int hwcap)
 {
-       int *hwcaps, i;
+       const int *hwcaps;
+       int i;
+
+       sr_spew("dev: %s: requesting hwcap %d", __func__, hwcap);
 
        if (!dev) {
                sr_err("dev: %s: dev was NULL", __func__);
-               return FALSE; /* TODO: SR_ERR_ARG. */
+               return FALSE;
        }
 
+       /*
+        * Virtual devices (which have dev->driver set to NULL) always say that
+        * they don't have the capability (they can't call hwcap_get_all()).
+        */
        if (!dev->driver) {
-               sr_err("dev: %s: dev->driver was NULL", __func__);
-               return FALSE; /* TODO: SR_ERR_ARG. */
+               sr_dbg("dev: %s: dev->driver was NULL, this seems to be "
+                      "a virtual device without capabilities", __func__);
+               return FALSE;
        }
 
        /* TODO: Sanity check on 'hwcap'. */
 
-       if (!(hwcaps = dev->driver->hwcap_get_all())) {
+       if (dev->driver->info_get(SR_DI_HWCAPS,
+                       (const void **)&hwcaps, NULL) != SR_OK) {
                sr_err("dev: %s: dev has no capabilities", __func__);
-               return FALSE; /* TODO: SR_ERR*. */
+               return FALSE;
        }
 
        for (i = 0; hwcaps[i]; i++) {
@@ -55,17 +55,17 @@ static GSList *devs = NULL;
  * TODO: Option to only scan for specific devices or device classes.
  *
- * @return SR_OK upon success, SR_ERR upon errors.
+ * @return SR_OK upon success, SR_ERR_BUG upon internal errors.
  */
 SR_API int sr_dev_scan(void)